function [phi, gam, ppsi, ll] = billda_e_step(document, alph, bet, ppie, label)
% [phi, gam] = lda_e_step(document,alph,bet)
% This function computes the m-step for the lda algorithm
% document - the count of words for one document rows = word counts
% gam = gamma parameter for this document
% alph, bet - current model parameters

index = find(document>0);
doc = full(document(index));
label = label(index,:);
missinglabel = sum(label,2) == 0;

nVocab = length(doc);
nWord = sum(doc);
nTopic = length(alph);
nLabel = length(ppie(:,1));

% ---------------------------------------------------------
% Initialize Variational Parameters: phi, gam, ppsi
% ---------------------------------------------------------
phi = row_normalize(repmat(1/nTopic,nVocab,nTopic) + rand(nVocab,nTopic));

ppsi = row_normalize(label);
ppsi(missinglabel,:) = repmat(normalize(sum(ppsi)),sum(missinglabel),1) + ...
    rand(sum(missinglabel),nLabel);
ppsi = row_normalize(ppsi);

gam = alph + repmat(nWord,nTopic,1)./nTopic;

% ---------------------------------------------------------
% Initialize the stopping criterion
% ---------------------------------------------------------
count = 0;
deltaPhi = inf;
deltaGam = inf;
deltaPsi = inf;

% ---------------------------------------------------------
% Prereplicate some matrices to save computations
% ---------------------------------------------------------
logpie = log(repmat(permute(ppie,[3,2,1]),[nVocab,1,1]));
repdoc = repmat(doc,1,nTopic); % not needed

% ---------------------------------------------------------
% Loop through this EM algorithm to estimate the 
% variational parameters
% ---------------------------------------------------------
while((deltaPhi>1e-5 | deltaGam>1e-5 | deltaPsi>1e-5))
    
    % Store the old variational parameters
    gam_old = gam;
    phi_old = phi;    
    ppsi_old = ppsi;
    
    % ---------------------------------------------------------
    % Run the E-step for phi and ppsi for fixed gamma
    % ---------------------------------------------------------
    
    % Replicate gamma to make an nWords x nTopics matrix
    
    
    %for n = 1:nVocab  % Without this loop the algorithm converges prematurely
    repgam = repmat(digamma(gam'),[nVocab,1]);
    repphi = repmat(phi,[1,1,nLabel]);
    
    % Fill in the missing values for ppsi using the expectation
    if(any(missinglabel))
        z= sum(repphi(missinglabel,:,:).*logpie(missinglabel,:,:),2);
        ppsi(missinglabel,:) = exp(squeeze(z));
    end
    ppsi = row_normalize(ppsi);
    reppsi=repmat(permute(ppsi,[1 3 2]),[1 nTopic 1]);
    
    % Recompute the expectation of phi
    phi = exp(repgam + log(bet(index,:)+realmin) + sum(reppsi.*logpie,3));
    phi = row_normalize(phi);
    
    % Recompute gamma for the estimated phi
    gam = alph + sum(repdoc.*phi)';
    
    %end
    % Compute the change in the variational parameters
    deltaPhi = norm(phi-phi_old)/norm(phi_old);
    deltaGam = norm(gam-gam_old)/norm(gam_old);
    deltaPsi = norm(ppsi - ppsi_old)/norm(ppsi_old);
end

% ---------------------------------------------------------
% Compute the log-likelihood for this parameter set
% ---------------------------------------------------------
[ll] = compute_billdaLL(document, alph, bet, gam, phi, ppsi, ppie);