function [ll] = compute_billdaLL(document, alph, bet, gam, phi, ppsi, ppie)
% [ll] = compute_ldaLL(document, alph, bet, gam, phi, lamda, ppie)
% Computes the log-likelihood of a document under the LDA model
% document - vector of word counts
% alph,b - alph & bet model parameters
% gam,phi,ppsi,ppie - gamma, phi,ppsi,ppie - variational model parameters


index = find(document>0);
document = document(index);
nVocab = length(document);
nWord = length(document);
nTopic = length(alph);
nLabel = length(ppie(:,1));

repdoc = repmat(document,1,nTopic);
repdocL = repmat(document,1,nLabel);
repgam = repmat(digamma(gam)',nVocab,1);


% Eq[log p(theta|alph)]
term(1) = gammaln(sum(alph))-sum(gammaln(alph))+ ...
    sum((alph-1).*(digamma(gam) - digammas(sum(gam))));

% Eq[log p(z|theta)]
term(2) = sum(sum(repdoc.*phi.*(repgam - digammas(sum(gam)))));

% Eq[log p(label|z;ppie)]
term(3) = sum(document.*diag(ppsi*log(ppie)*phi'));

% Eq[log p(word|z;bet)]
term(4) = sum(sum(phi.*repdoc.*log(bet(index,:)+realmin)));

% Entropy Terms
term(5) = -gammaln(sum(gam)) + sum(gammaln(gam)) -...
    sum((gam-1).*(digamma(gam) - digammas(sum(gam))));

term(6) = -sum(sum(repdoc.*phi.*log(phi+realmin),2));

term(7) = -sum(sum(repdocL.*ppsi.*log(ppsi+realmin),2));

ll = sum(term);
